--=========================================================
-- 
-- セーブ機能実験用アプリ
--
-- このファイルの下部にある toSave関数とtoLoad関数が主役です。
--
-- （ちなみになんで「to」をつけてるかと言うと、言語の予約語と被らないようにするためです）
-- 
--=========================================================

require ( "playerChar" )
require ( "zunda" )
utils = require("utils")

--=========================================================
-- 初期設定
--=========================================================
function love.load ()
	
	fonts = {}
	fonts.default         = love.graphics.newFont("PixelMplus12-Regular.ttf", 50)
	fonts.message         = love.graphics.newFont("PixelMplus12-Regular.ttf", 30)
	love.graphics.setFont ( fonts.default )

	spriteSheet_zunko = love.graphics.newImage ( "symbol_zunko.png" )
	spriteSheet_zunda = love.graphics.newImage ( "zunda.png" )
	
	sounds = {}
	sounds[ "take" ] = love.audio.newSource ( "ステータス変化系＿パワーアップ.wav", "static" )
	
	-- 定数
	SPRITE_WIDTH  = 32
	SPRITE_HEIGHT = 32	
	WIDTH  = 640
	HEIGHT = 480
	UP    = 1
	RIGHT = 2
	DOWN  = 3
	LEFT  = 4
	
	-- FPS安定化処理の設定
	love.window.setVSync(1)
	min_dt = 1/60
	next_time = love.timer.getTime()	
	
	love.window.setMode(WIDTH, HEIGHT)
	love.window.setTitle ("セーブ機能サンプル")
	
	
	player = PlayerChar.new()
	
	math.randomseed(os.time())

	zunda = nil
	zundaRespawnWait = 0
	zundaNum = 0
	
	nextZundaX = math.random(WIDTH)
	nextZundaY = math.random(HEIGHT)
	
	systemMessage = { 0, "" }
	
end

--=========================================================
-- 毎フレームの更新処理
--=========================================================
function love.update ( dt )
	next_time = next_time + min_dt -- FPS安定化用
	
	player:update()

	
	if ( zunda == nil ) then
		
		zundaRespawnWait = zundaRespawnWait - 1
		if ( zundaRespawnWait <= 0 ) then
			zunda = Zunda.new ( { x = nextZundaX, y = nextZundaY } )
			zundaRespawnWait = 0
			nextZundaX = math.random(WIDTH)
			nextZundaY = math.random(HEIGHT)
		end
		
	else

		zunda:update()

		if ( zunda.isTaken ) then
			playSound ( sounds["take"] )
			zundaNum = zundaNum + 1
			zunda = nil
			zundaRespawnWait = 20
		end
	end
	
	if ( systemMessage[1] > 0 ) then
		systemMessage[1] = systemMessage[1] - 1
	end
	
end

--=========================================================
-- 毎フレームの描画処理
--=========================================================
function love.draw ()
	
	love.graphics.clear(0.8,0.5,0.5,1)
	love.graphics.setColor(1,1,1,1)

	if ( zunda ~= nil ) then zunda:draw() end
	
	player:draw()

	love.graphics.setColor(0,0,0,1)
	love.graphics.setFont ( fonts.default )
	
	local zundaNumText = tostring(zundaNum) .. "ずん"
	love.graphics.print ( zundaNumText, WIDTH - fonts.default:getWidth(zundaNumText) - 20  , 10 )
	
	
	if ( systemMessage[1] > 0 ) then

		love.graphics.setColor(1,1,0,1)
		love.graphics.setFont ( fonts.message )
		love.graphics.print ( systemMessage[2], 10  , HEIGHT - fonts.message:getHeight( systemMessage[2] ) - 10 )
	end
	
	-- FPS安定化処理
	local cur_time = love.timer.getTime()
	if next_time <= cur_time then
	  next_time = cur_time
	  return
	end
	love.timer.sleep(next_time - cur_time)	
end

--=========================================================
-- サウンドを鳴らす
--=========================================================
volumeSOUND = 1.0
function playSound ( soundSource ) 

	if ( soundSource ) then
		
		soundSource:setVolume ( volumeSOUND )
		
		if ( soundSource:isPlaying() ) then
			soundSource:stop()
		end
		love.audio.play ( soundSource )	
	
	else
		print ( "soundSource is null." )
	end
	
end


--=========================================================
-- キー押下
-- セーブ/ロード発動
--=========================================================
function love.keypressed ( key )
	
	-- SHIFTを押しながら 1〜5キー で ロード
	if ( love.keyboard.isDown ("lshift") or love.keyboard.isDown ("rshift") ) then

		if ( key == "1" ) then toLoad(key) end
		if ( key == "2" ) then toLoad(key) end
		if ( key == "3" ) then toLoad(key) end
		if ( key == "4" ) then toLoad(key) end
		if ( key == "5" ) then toLoad(key) end
	
	-- 1〜5キー で セーブ
	else

		if ( key == "1" ) then toSave(key) end
		if ( key == "2" ) then toSave(key) end
		if ( key == "3" ) then toSave(key) end
		if ( key == "4" ) then toSave(key) end
		if ( key == "5" ) then toSave(key) end
	end
end


--=========================================================
-- メッセージ表示
--=========================================================
showMessage = function ( message )
	
	systemMessage[1] = 60 -- 表示する時間（60フレーム）	
	systemMessage[2] = message
	
end


--=========================================================
-- セーブデータ書き込み
-- 引数はスロット番号 セーブファイルのファイル名につける
--=========================================================
toSave = function ( slot ) 
	
	-- ダミー込みで要素100個の配列を作り、ランダムな値で埋める
	local saveData = {}
	for i = 1, 100, 1 do
		
		local data = math.random ( 0,999 )
		saveData [ #saveData + 1 ] = data
	end
	
	--[[
		保存する情報メモ（もっと手抜きしてもいいし、もっと細かくしてもいい）
		
		・自キャラのx,y
		・自キャラの向き
		・ずんだ餅のx,y
		・取得ずんだ数
		・ずんだ餅出現前のインターバル（「取った感」を演出するため、取ってから次のずんだ餅が出現するまで少しタイムラグを入れている）
		・次のずんだ餅出現位置
	]]--
	
	-- 用意しておいた配列の中に、位置を適当に決めて各情報を入れていく
	saveData [ 32 ] = player.x
	saveData [ 51 ] = player.y
	saveData [ 70 ] = player.direction
	
	-- ずんだ餅は取った直後は消えているので、存在の有無を確認している
	if ( zunda ~= nil ) then
		saveData [ 85 ] = zunda.x
		saveData [ 7  ] = zunda.y
	else
		saveData [ 85 ] = 0
		saveData [ 7  ] = 0
	end
	saveData [ 50 ] = zundaNum
	saveData [ 2  ] = zundaRespawnWait
	saveData [ 41 ] = nextZundaX
	saveData [ 65 ] = nextZundaY
	
	
	-- チェックサムを計算
	local sum = 0
	for i = 1, #saveData, 1 do
		sum = sum + saveData[i]
	end
	sum = sum - saveData[66]  -- 配列の66番目の要素をチェックサムとするので合計から差し引いている
	local checkSum = sum%1000 -- 値を1000で割った余りにしている
	
	saveData[66] = checkSum -- 66番という数字に深い意味はない


	-- セーブファイルに書き込む文字列を作っていく
	local saveDataSerial = ""
	
	-- セーブ情報を書き込んだ配列を "," で区切った文字列にしている
	for i = 1, #saveData, 1 do
		
		saveDataSerial = saveDataSerial..saveData[i]
		
		if ( i < #saveData ) then
			saveDataSerial = saveDataSerial..","
		end
	end
		
	-- ファイルに出力している
	utils.writeFile ( "save"..slot, saveDataSerial )
	
	-- セーブしましたというメッセージを画面に表示する
	showMessage ( "セーブ [" .. slot .. "]" )
	
end


--=========================================================
-- セーブデータ読み込み
-- 引数はスロット番号
--=========================================================
toLoad = function ( slot ) 
	
	-- 指定されたスロット番号のセーブファイルを読み込む
	local saveDataSerial = utils.readFile ( "save"..slot )
	
	-- "," 区切りの文字列（の筈）なので、それを配列に戻す
	local saveData = utils.split ( saveDataSerial, ",")
	
	-- 要素数は100個のはず。そうじゃないならエラーを出す（→else節へ）
	if ( #saveData == 100 ) then
		
		-- チェックサムを確認
		-- 読み込んだセーブデータのチェックサムを計算
		local sum = 0
		for i = 1, #saveData, 1 do
			sum = sum + tonumber(saveData[i])
		end
		sum = sum - tonumber(saveData[66]) -- 66番目の要素がチェックサムってことにしている
		local checkSum = sum%1000 -- 1000で割った余りということにしている
		
		-- チェックサムが不一致だったらエラーを出す
		if ( tonumber(saveData[66]) ~= checkSum ) then
			
			showMessage ( "チェックサムが合わない [" .. slot .. "]" )
		
		-- チェックサムが合っている場合はロード処理を実行
		else
			
			-- セーブデータを作ったときの逆の手順。要素番号を自分で決めた仕様の通りに。
			player.x          = tonumber ( saveData [ 32 ] ) 
			player.y          = tonumber ( saveData [ 51 ] )
			player.direction  = tonumber ( saveData [ 70 ] )
			zundaNum          = tonumber ( saveData [ 50 ] )
			nextZundaX        = tonumber ( saveData [ 41 ] )
			nextZundaY        = tonumber ( saveData [ 65 ] )
			zundaRespawnWait  = tonumber ( saveData [ 2  ] )
			
			-- ずんだ餅が出現待ちの状態かどうか？ 
			if ( zundaRespawnWait > 0 ) then
				
				zunda = nil -- ずんだ無き世界とする
				
			-- ずんだ餅を再現
			else
				zunda = Zunda.new ( { x = tonumber(saveData[85]), y = tonumber(saveData[7]) } )
			end
			
			-- ロード処理が完了した旨のメッセージを画面に表示している
			showMessage ( "ロード [" .. slot .. "]" )
			
		end
		
		
	-- セーブデータがおかしかった場合のエラーメッセージ
	else
		showMessage ( "セーブデータが存在しないか壊れています [" .. slot .. "]" )
	end

end

